from xendit.models._base_model import BaseModel
from xendit.models._base_query import BaseQuery


class DirectDebitCustomerAddress(BaseModel):
    """Address class of Customer in Direct Debit (API Reference: Direct Debit)

    Use this to initialize create_customer

    Attributes:
      - country (str)

    Optional Attributes:
      - street_line1 (str)
      - street_line2 (str)
      - city (str)
      - province (str)
      - state (str)
      - postal_code (str)
    """

    country: str

    # Optional
    street_line1: str
    street_line2: str
    city: str
    province: str
    state: str
    postal_code: str

    class Query(BaseQuery):
        """Address class of Customer in Direct Debit (API Reference: Direct Debit)

        Use this to initialize create_customer

        Attributes:
          - country (str)

        Optional Attributes:
          - street_line1 (str)
          - street_line2 (str)
          - city (str)
          - province (str)
          - state (str)
          - postal_code (str)
        """

        country: str

        # Optional
        street_line1: str
        street_line2: str
        city: str
        province: str
        state: str
        postal_code: str



from xendit.models._base_model import BaseModel


class VirtualAccountBank(BaseModel):
    """Bank class for Virtual Account (API Reference: Virtual Account)

    Attributes:
      - name (str)
      - code (str)
    """

    name: str
    code: str



from .credit_card_charge_address import CreditCardChargeAddress

from xendit.models._base_model import BaseModel
from xendit.models._base_query import BaseQuery


class CreditCardChargeBillingDetails(BaseModel):
    """BillingDetails class of Charge (API Reference: Credit Card)"""

    class Query(BaseQuery):
        """BillingDetails class of Charge Query (API Reference: Credit Card)

        Use this for initialize create_authorization and create_charge

        Attributes:
          - given_names (str)
          - address (CreditCardCharge.Address)

        Optional Attributes:
          - middle_name (str)
          - surname (str)
          - email (str)
          - mobile_number (str)
          - phone_number (str)

        """

        given_names: str
        address: CreditCardChargeAddress

        # Optional
        middle_name: str
        surname: str
        email: str
        mobile_number: str
        phone_number: str



from xendit._api_requestor import _APIRequestor
from xendit._extract_params import _extract_params

from xendit.xendit_error import XenditError

from .ewallet_type import EWalletType
from .ovo import OVOPayment, OVOPaymentStatus
from .dana import DANAPayment, DANAPaymentStatus
from .linkaja import LinkAjaPayment, LinkAjaPaymentStatus, LinkAjaItem


class EWallet:
    """EWallet class (API Reference: eWallets)

    Related Classes:
      - OVOPayment
      - OVOPaymentStatus
      - DANAPayment
      - DANAPaymentStatus
      - LinkAjaPayment
      - LinkAjaPaymentStatus

    Static Methods:
      - EWallet.create_ovo_payment (API Reference: /Create Payment)
      - EWallet.create_dana_payment (API Reference: /Create Payment)
      - EWallet.create_linkaja_payment (API Reference: /Create Payment)
      - EWallet.get_payment_statis (API Reference: /Get Payment Status)

    Static Methods for Object Creation:
      - CreditCard.helper_create_linkaja_item (For create_linkaja_payment)
    """

    @staticmethod
    def helper_create_linkaja_item(*, id, name, price, quantity, **kwargs):
        """Construct Installments Object for Charge

        Args:
          - id (str)
          - name (str)
          - price (int)
          - quantity (int)

        Return:
          - LinkajaItem
        """
        params = locals()
        del params["kwargs"]

        return LinkAjaItem.Query(**params)

    @staticmethod
    def create_ovo_payment(
        *,
        external_id,
        amount,
        phone,
        for_user_id=None,
        x_idempotency_key=None,
        x_api_version=None,
        **kwargs,
    ):
        """Send POST Request to create OVO Payment (API Reference: eWallets/Create Payment)

        Args:
          - external_id (str)
          - amount (int)
          - phone (str)
          - **for_user_id (str) (XenPlatforms only)
          - **x_idempotency_key (str)
          - **x_api_version (str)

        Returns:
          OVOPayment

        Raises:
          XenditError

        """
        url = "/ewallets"
        headers, body = _extract_params(
            locals(),
            func_object=EWallet.create_ovo_payment,
            headers_params=["for_user_id", "x_idempotency_key", "x_api_version"],
        )
        body["ewallet_type"] = "OVO"
        kwargs["headers"] = headers
        kwargs["body"] = body

        resp = _APIRequestor.post(url, **kwargs)
        if resp.status_code >= 200 and resp.status_code < 300:
            return OVOPayment(**resp.body)
        else:
            raise XenditError(resp)

    @staticmethod
    def create_dana_payment(
        *,
        external_id,
        amount,
        callback_url,
        redirect_url,
        expiration_date=None,
        for_user_id=None,
        x_idempotency_key=None,
        x_api_version=None,
        **kwargs,
    ):
        """Send POST Request to create DANA Payment (API Reference: eWallets/Create Payment)

        Args:
          - external_id (str)
          - amount (int)
          - callback_url (str)
          - redirect_url (str)
          - **expiration_date (str)
          - **for_user_id (str) (XenPlatforms only)
          - **x_idempotency_key (str)
          - **x_api_version (str)

        Returns:
          DANAPayment

        Raises:
          XenditError

        """
        url = "/ewallets"
        headers, body = _extract_params(
            locals(),
            func_object=EWallet.create_dana_payment,
            headers_params=["for_user_id", "x_idempotency_key", "x_api_version"],
        )
        body["ewallet_type"] = "DANA"
        kwargs["headers"] = headers
        kwargs["body"] = body

        resp = _APIRequestor.post(url, **kwargs)
        if resp.status_code >= 200 and resp.status_code < 300:
            return DANAPayment(**resp.body)
        else:
            raise XenditError(resp)

    @staticmethod
    def create_linkaja_payment(
        *,
        external_id,
        phone,
        amount,
        items,
        callback_url,
        redirect_url,
        for_user_id=None,
        x_idempotency_key=None,
        x_api_version=None,
        **kwargs,
    ):
        """Send POST Request to create LinkAja Payment (API Reference: eWallets/Create Payment)

        Args:
          - external_id (str)
          - phone (str)
          - amount (int)
          - items (LinkAjaItem.Query[])
          - callback_url (str)
          - redirect_url (str)
          - **for_user_id (str) (XenPlatforms only)
          - **x_idempotency_key (str)
          - **x_api_version (str)

        Returns:
          LinkAjaPayment

        Raises:
          XenditError

        """
        url = "/ewallets"
        headers, body = _extract_params(
            locals(),
            func_object=EWallet.create_linkaja_payment,
            headers_params=["for_user_id", "x_idempotency_key", "x_api_version"],
        )
        body["ewallet_type"] = "LINKAJA"
        kwargs["headers"] = headers
        kwargs["body"] = body

        resp = _APIRequestor.post(url, **kwargs)
        if resp.status_code >= 200 and resp.status_code < 300:
            return LinkAjaPayment(**resp.body)
        else:
            raise XenditError(resp)

    @staticmethod
    def get_payment_status(
        *, external_id, ewallet_type, for_user_id=None, x_api_version=None, **kwargs
    ):
        """Get payment status of given external id (API Reference: eWallets/Get Payment Status)

        Args:
          - external_id (str)
          - ewallet_type (EWalletType)
          - **for_user_id (str) (XenPlatforms only)
          - **x_api_version (str)

        Returns:
          - OVOPaymentStatus (For EWalletType.OVO)
          - DANAPaymentStatus (For EWalletType.DANA)
          - LinkAjaPaymentStatus (For EWalletType.LINKAJA)

        Raises:
          XenditError

        """
        parsed_ewallet_type = EWallet._parse_ewallet_type(ewallet_type)
        url = f"/ewallets?external_id={external_id}&ewallet_type={parsed_ewallet_type}"
        headers, _ = _extract_params(
            locals(),
            func_object=EWallet.create_linkaja_payment,
            headers_params=["for_user_id", "x_api_version"],
        )
        kwargs["headers"] = headers

        resp = _APIRequestor.get(url, **kwargs)
        if resp.status_code >= 200 and resp.status_code < 300:
            if ewallet_type == EWalletType.OVO:
                return OVOPaymentStatus(**resp.body)
            elif ewallet_type == EWalletType.DANA:
                return DANAPaymentStatus(**resp.body)
            elif ewallet_type == EWalletType.LINKAJA:
                return LinkAjaPaymentStatus(**resp.body)
        else:
            raise XenditError(resp)

    @staticmethod
    def _parse_ewallet_type(ewallet_type):
        try:
            return ewallet_type.name
        except AttributeError:
            return ewallet_type



import base64
import requests
import xendit
import sys
from xendit.network import XenditResponse

if sys.version_info[1] < 8:
    from importlib_metadata import version
else:
    from importlib.metadata import version


class _APIRequestor:
    @staticmethod
    def get(url, **kwargs):
        return _APIRequestor._request("GET", url, **kwargs)

    @staticmethod
    def post(url, **kwargs):
        return _APIRequestor._request("POST", url, **kwargs)

    @staticmethod
    def patch(url, **kwargs):
        return _APIRequestor._request("PATCH", url, **kwargs)

    @staticmethod
    def _request(
        method,
        url,
        api_key=None,
        base_url=None,
        http_client=requests,
        headers={},
        body={},
        params={},
    ):
        """Send HTTP Method to given url

        Args:
          - method (str): HTTP Method that will be send
          - url (str): URL Directory that will be searched (not including base_url)
          - **api_key (string): API Key from xendit instance. Default to config if not provided
          - **base_url (string): Base url of the API. Default to config if not provided
          - **http_client (HTTPClientInterface): HTTP Client that adhere to HTTPClientInterface. Default to config if not provided
          - **headers: Headers of the request
          - **body: Body of the request. Only used on POST and PATCH request
          - **params: Parameters of the request. Only used on GET request
        """
        if api_key is None:
            api_key = xendit.api_key
        if base_url is None:
            base_url = xendit.base_url
        url = base_url + url

        headers = _APIRequestor._add_default_headers(api_key, headers)
        if method == "GET":
            resp = http_client.request(method, url, headers=headers, params=params)
        else:
            resp = http_client.request(method, url, headers=headers, json=body)
        return XenditResponse(resp.status_code, resp.headers, resp.json())

    @staticmethod
    def _add_default_headers(api_key, headers):
        headers["Content-type"] = "application/json"
        headers["Authorization"] = f"Basic {_APIRequestor._generate_auth(api_key)}"
        headers["xendit-lib"] = "python"
        headers["xendit-lib-ver"] = version("xendit-python")
        return headers

    @staticmethod
    def _generate_auth(api_key):
        auth_pair = api_key + ":"
        auth_base64 = base64.b64encode(auth_pair.encode())
        return auth_base64.decode("utf-8")



from .direct_debit_accessible_account import DirectDebitAccessibleAccount
from .direct_debit_card_link import DirectDebitCardLink
from .direct_debit_online_banking_link import DirectDebitOnlineBankingLink
from .direct_debit_token import DirectDebitToken



import abc
import requests


class HTTPClientInterface(metaclass=abc.ABCMeta):
    """Interface for HTTP Client. Inject it to your xendit instance to use it."""

    @staticmethod
    def request(method, url, **kwargs) -> requests.Response:
        """
        Args:
          - method (str): HTTP Method that will be sent. For Xendit we will mainly use "GET", "POST", "PATCH"
          - url (str): URL for the request
          - **headers (dict): HTTP Headers to send with the request
          - **json (dict): Body that will be send with the request

        Returns:
          requests.Response
        """
        raise NotImplementedError



from xendit.models._base_model import BaseModel


class OVOPaymentStatus(BaseModel):
    """Payment Status for OVO (API Reference: eWallets)

    Attributes:
      - amount (str)
      - business_id (str)
      - ewallet_type (str)
      - external_id (str)
      - status (str)
      - transaction_date (str) (ISO 8601 Date)
    """

    amount: str
    business_id: str
    ewallet_type: str
    external_id: str
    status: str
    transaction_date: str



from .direct_debit_payment_method import DirectDebitPaymentMethod
from .direct_debit_payment_method_properties import DirectDebitPaymentMethodProperties
from .direct_debit_payment_method_type import DirectDebitPaymentMethodType



from xendit.models._base_model import BaseModel
from .credit_card_charge_installment import CreditCardChargeInstallment
from .credit_card_charge_promotion import CreditCardChargePromotion


class CreditCardCharge(BaseModel):
    """Charge class (API Reference: Credit Card)

    Attributes:
      - created (str)
      - charge_type (str)
      - business_id (str)
      - authorized_amount (float)
      - external_id (str)
      - card_type (str)
      - merchant_id (str)
      - masked_card_number (str)
      - charge_type (str)
      - card_brand (str)
      - bank_reconciliation_id (str)
      - id (str)

    Optional Attributes:
      - eci (str)
      - capture_amount (float)
      - failure_reason (str)
      - approval_code (str)
      - merchant_reference_code (str)
      - descriptor (str)
      - currency (str)
      - mid_label (str)
      - promotion (CreditCardChargePromotion)
      - installment (CreditCardChargeInstallment)
    """

    created: str
    charge_type: str
    business_id: str
    authorized_amount: float
    external_id: str
    card_type: str
    merchant_id: str
    masked_card_number: float
    charge_type: str
    card_brand: str
    bank_reconciliation_id: str
    id: str

    # Optional
    eci: str
    capture_amount: float
    failure_reason: str
    approval_code: str
    merchant_reference_code: str
    descriptor: str
    currency: str
    mid_label: str
    promotion: CreditCardChargePromotion
    installment: CreditCardChargeInstallment



from xendit.models._base_model import BaseModel


class DANAPayment(BaseModel):
    """Payment detail for DANA (API Reference: eWallets)

    Attributes:
      - external_id (str)
      - amount (int)
      - checkout_url (str)
      - ewallet_type (str)
    """

    external_id: str
    amount: int
    checkout_url: str
    ewallet_type: str



from xendit.models._base_model import BaseModel
from xendit.models._base_query import BaseQuery


class CreditCardChargePromotion(BaseModel):
    """Promotion class of Credit Card Charge (API Reference: Credit Card)

    Optional Attributes:
        - reference_id (str)
        - original_amount (float)
    """

    # Optional
    reference_id: str
    original_amount: int

    class Query(BaseQuery):
        """Promotion class of Credit Card Charge Query (API Reference: Credit Card)

        Used for create_authorization and create_charge

        Optional Attributes:
          - reference_id (str)
          - original_amount (float)
        """

        # Optional
        reference_id: str
        original_amount: int



from xendit.models._base_model import BaseModel


class InvoiceRetailOutlet(BaseModel):
    """Retail Outlet data detail in Invoice (API Reference: Invoice)

    Attributes:
      - retail_outlet_name (str)
      - payment_code (str)
      - transfer_amount (int)
    """

    retail_outlet_name: str
    payment_code: str
    transfer_amount: int



from xendit.models._base_model import BaseModel
from xendit.models._base_query import BaseQuery


class CreditCardChargeAddress(BaseModel):
    """Address class of Charge Billing Details (API Reference: Credit Card)"""

    class Query(BaseQuery):
        """Address class of Charge Billing Details Query (API Reference: Credit Card)

        Use this for initialize create_authorization and create_charge

        Attributes:
          - country (str)

        Optional Attributes:
          - street_line1 (str)
          - street_line2 (str)
          - city (str)
          - province (str)
          - state (str)
          - postal_code (str)
          - description (str)
        """

        country: str

        # Optional
        street_line1: str
        street_line2: str
        city: str
        province: str
        state: str
        postal_code: str
        description: str



from xendit.models._base_model import BaseModel


class DirectDebitToken(BaseModel):
    """Token class in Direct Debit (API Reference: Direct Debit)

    Attributes:
      - id (str)
      - customer_id (str)
      - channel_code (str)
      - status (str)
      - metadata (dict)

    Optional Attributes:
      - authorizer_url (str)

    """

    id: str
    customer_id: str
    channel_code: str
    status: str
    metadata: dict

    # Optional
    authorizer_url: str



from typing import List
from .direct_debit_customer_address import DirectDebitCustomerAddress

from xendit.models._base_model import BaseModel


class DirectDebitCustomer(BaseModel):
    """Customer class in Direct Debit (API Reference: Direct Debit)

    Attributes:
      - id (str)
      - reference_id (str)
      - mobile_number (str)
      - email (str)
      - given_names (str)
      - middle_name (str)
      - surname (str)
      - description (str)
      - phone_number (str)
      - nationality (str)
      - addresses (DirectDebitCustomerAddress[])
      - date_of_birth (str)
      - metadata (dict)

    """

    id: str
    reference_id: str
    mobile_number: str
    email: str
    given_names: str
    middle_name: str
    surname: str
    description: str
    phone_number: str
    nationality: str
    addresses: List[DirectDebitCustomerAddress]
    date_of_birth: str
    metadata: dict



from typing import List
from xendit.models._base_model import BaseModel


class CreditCardPromotion(BaseModel):
    """Promotion class (API Reference: Credit Card)

    Attributes:
      - id (str)
      - business_id (str)
      - status (str)
      - reference_id (str)
      - description (str)
      - promo_code (str)
      - bin_list (str[])
      - channel_code (str)
      - discount_percent (float)
      - discount_amount (float)
      - currency (str)
      - start_time (str)
      - end_time (str)
    """

    id: str
    business_id: str
    status: str
    reference_id: str
    description: str
    promo_code: str
    bin_list: List[str]
    channel_code: str
    discount_percent: float
    discount_amount: float
    currency: str
    start_time: str
    end_time: str



import requests

from ._xendit_param_injector import _XenditParamInjector

from .network import HTTPClientInterface


class Xendit:
    """Xendit instance. Initialize this with your API Key."""

    def __init__(
        self,
        api_key,
        base_url="https://api.xendit.co/",
        http_client: HTTPClientInterface = requests,
    ):
        injected_params = (api_key, base_url, http_client)
        param_injector = _XenditParamInjector(injected_params)

        self.Balance = param_injector.instantiate_balance()
        self.CreditCard = param_injector.instantiate_credit_card()
        self.DirectDebit = param_injector.instantiate_direct_debit()
        self.Disbursement = param_injector.instantiate_disbursement()
        self.EWallet = param_injector.instantiate_ewallet()
        self.Invoice = param_injector.instantiate_invoice()
        self.RecurringPayment = param_injector.instantiate_recurring_payment()
        self.RetailOutlet = param_injector.instantiate_retail_outlet()
        self.VirtualAccount = param_injector.instantiate_virtual_account()



from xendit.models._base_model import BaseModel


class CreditCardReverseAuthorization(BaseModel):
    """Reverse Authorization class (API Reference: Credit Card)

    Attributes:
      - status (str)
      - currency (str)
      - credit_card_charge_id (str)
      - business_id (str)
      - external_id (str)
      - amount (int)
      - created (str)
      - id (str)

    Optional Attributes:
      - failure_reason (str)
    """

    status: str
    currency: str
    credit_card_charge_id: str
    business_id: str
    external_id: str
    amount: int
    created: str
    id: str

    # Optional
    failure_reason: str



from xendit.models._base_model import BaseModel
from xendit.models._base_query import BaseQuery


class DirectDebitOnlineBankingLink(BaseModel):
    """Online Banking Linking class in Direct Debit Tokenization (API Reference: Direct Debit)

    Use this for initialize_tokenization

    Attributes:
      - account_details (str)
      - account_hash (str)
      - account_type (str)
      - currency (str)
      - description (str)

    Parameters Attributes (initialize_tokenization):
      - success_redirect_url (str)
      - failure_redirect_url (str)
      - callback_url (str)

    """

    account_details: str
    account_hash: str
    account_type: str
    currency: str
    description: str

    class Query(BaseQuery):
        """Online Banking Linking class in Direct Debit Tokenization (API Reference: Direct Debit)

        Use this for initialize_tokenization

        Attributes:
          - success_redirect_url (str)
          - failure_redirect_url (str)
          - callback_url (str)

        """

        success_redirect_url: str
        failure_redirect_url: str
        callback_url: str



from .credit_card_charge import CreditCardCharge
from .credit_card_charge_address import CreditCardChargeAddress
from .credit_card_charge_billing_details import CreditCardChargeBillingDetails
from .credit_card_charge_installment import CreditCardChargeInstallment
from .credit_card_charge_promotion import CreditCardChargePromotion



from .customer import DirectDebitCustomer, DirectDebitCustomerAddress
from .payment import (
    DirectDebitBasket,
    DirectDebitPayment,
)
from .paymentmethod import (
    DirectDebitPaymentMethodProperties,
    DirectDebitPaymentMethod,
)
from .token import (
    DirectDebitCardLink,
    DirectDebitOnlineBankingLink,
    DirectDebitToken,
    DirectDebitAccessibleAccount,
)
from xendit.models._base_model import BaseModel

from xendit._api_requestor import _APIRequestor
from xendit._extract_params import _extract_params

from xendit.xendit_error import XenditError


class DirectDebit(BaseModel):
    """DirectDebit class (API Reference: Direct Debit)

    Related Classes:
      - DirectDebitCustomer
      - DirectDebitCustomerAddress
      - DirectDebitCardLink
      - DirectDebitOnlineBankingLinking
      - DirectDebitToken
      - DirectDebitAccessibleAccount
      - DirectDebitPaymentMethod
      - DirectDebitPaymentMethodProperties
      - DirectDebitBasket
      - DirectDebitPayment

    Static Methods:
      - DirectDebit.create_customer (API Reference: /Create Customer)
      - DirectDebit.get_customer_by_ref_id (API Reference: /Get Customer by Reference ID)
      - DirectDebit.initialize_tokenization (API Reference: /Initialize Linked Account Tokenization)
      - DirectDebit.validate_token_otp (API Reference: /Validate OTP for Linked Account Token)
      - DirectDebit.get_accessible_account_by_token (API Reference: /Retrieve Accessible Accounts by Linked Account Token)
      - DirectDebit.create_payment_method (API Reference: /Create Payment Method)
      - DirectDebit.get_payment_method_by_customer_id (API Reference: /Get Payment Methods by Customer ID)
      - DirectDebit.create_payment (API Reference: /Create Direct Debit Payment)
      - DirectDebit.validate_payment_otp (API Reference: /Validate OTP for Direct Debit Payment)
      - DirectDebit.get_payment_status (API Reference: /Get Direct Debit Payment Status by ID)
      - DirectDebit.get_payment_status_by_ref_id (API Reference: /Get Direct Debit Payment Status by Reference ID)

    Static Methods for Object Creation:
      - DirectDebit.helper_create_customer_address (For Address in create_customer)
      - DirectDebit.helper_create_online_banking_link (For OnlineBankingLink in initialize_tokenization)
      - DirectDebit.helper_create_card_link (For CardLink in initialize_tokenization)
      - DirectDebit.helper_create_payment_method_properties (For Properties in create_payment_method)
      - DirectDebit.helper_create_basket (For Basket in create_payment)
    """

    @staticmethod
    def helper_create_customer_address(
        *,
        country,
        street_line1=None,
        street_line2=None,
        city=None,
        province=None,
        state=None,
        postal_code=None,
        **kwargs,
    ):
        """Construct Address Object for DirectDebit Customer

        Args:
          - country (str)
          - **street_line1 (str)
          - **street_line2 (str)
          - **city (str)
          - **province (str)
          - **state (str)
          - **postal_code (str)

        Return:
          - DirectDebitCustomerAddress
        """
        params = locals()
        del params["kwargs"]

        return DirectDebitCustomerAddress.Query(**params)

    @staticmethod
    def helper_create_online_banking_link(
        *, success_redirect_url, failure_redirect_url=None, callback_url=None, **kwargs,
    ):
        """Construct OnlineBankingLink Object for DirectDebit Token

        Args:
          - success_redirect_url (str)
          - failure_redirect_url (str)
          - callback_url (str)

        Return:
          - DirectDebitOnlineBankingLink
        """
        params = locals()
        del params["kwargs"]

        return DirectDebitOnlineBankingLink.Query(**params)

    @staticmethod
    def helper_create_card_link(
        *, account_mobile_number, card_last_four, card_expiry, account_email, **kwargs,
    ):
        """Construct CardLink Object for DirectDebit Token

        Args:
          - account_mobile_number (str)
          - card_last_four (str)
          - card_expiry (str)
          - account_email (str)

        Return:
          - DirectDebitCardLink.Query
        """
        params = locals()
        del params["kwargs"]

        return DirectDebitCardLink.Query(**params)

    def helper_create_payment_method_properties(*, id, **kwargs):
        """Construct Properties Object for DirectDebit Payment Methods

        Args:
          - id (str)

        Return:
          - DirectDebitPaymentMethodProperties.Query
        """
        params = locals()
        del params["kwargs"]

        return DirectDebitPaymentMethodProperties.Query(**params)

    @staticmethod
    def helper_create_basket(
        *,
        reference_id,
        name,
        market,
        type,
        description=None,
        category=None,
        sub_category=None,
        price=None,
        url=None,
        metadata=None,
        quantity=None,
        **kwargs,
    ):
        """Construct Basket object for DirectDebit payment

        Args:
          - reference_id (str)
          - name (str)
          - market (str)
          - type (str)
          - **description (str)
          - **category (str)
          - **sub_category (str)
          - **price (str)
          - **url (str)
          - **metadata (str)
          - **quantity (str)

        Return:
          - DirectDebitBasket.Query
        """
        params = locals()
        del params["kwargs"]

        return DirectDebitBasket.Query(**params)

    @staticmethod
    def create_customer(
        *,
        reference_id,
        given_names,
        mobile_number=None,
        email=None,
        middle_name=None,
        surname=None,
        description=None,
        phone_number=None,
        nationality=None,
        addresses=None,
        date_of_birth=None,
        metadata=None,
        x_idempotency_key=None,
        for_user_id=None,
        x_api_version=None,
        **kwargs,
    ):
        """Send POST Request to create Customer (API Reference: Direct Debit/Create Customer)

        Args:
          - reference_id (str)
          - given_names (str)
          - **mobile_number (str) (Either mobile_number or email must be present)
          - **email (Either mobile_number or email must be present)
          - **middle_name (str)
          - **surname (str)
          - **description (str)
          - **phone_number (str)
          - **nationality (str)
          - **addresses (DirectDebitCustomerAddress[])
          - **date_of_birth (str)
          - **metadata (dict)
          - **for_user_id (str)
          - **x_idempotency_key (str)
          - **x_api_version (str)

        Returns:
          DirectDebitCustomer

        Raises:
          XenditError

        """
        url = "/customers"
        headers, body = _extract_params(
            locals(),
            func_object=DirectDebit.create_customer,
            headers_params=["for_user_id", "x_idempotency_key", "x_api_version"],
        )
        kwargs["headers"] = headers
        kwargs["body"] = body

        resp = _APIRequestor.post(url, **kwargs)
        if resp.status_code >= 200 and resp.status_code < 300:
            return DirectDebitCustomer(**resp.body)
        else:
            raise XenditError(resp)

    @staticmethod
    def get_customer_by_ref_id(
        *, reference_id, for_user_id=None, x_api_version=None, **kwargs,
    ):
        """Get Customer by Reference ID (API Reference: Direct Debit/Get Customer by Reference ID)

        Args:
          - reference_id (str)
          - **for_user_id (str)
          - **x_api_version (str)

        Returns:
          DirectDebitCustomer

        Raises:
          XenditError

        """
        url = f"/customers?reference_id={reference_id}"
        headers, _ = _extract_params(
            locals(),
            func_object=DirectDebit.get_customer_by_ref_id,
            headers_params=["for_user_id", "x_api_version"],
            ignore_params=["reference_id"],
        )
        kwargs["headers"] = headers

        resp = _APIRequestor.get(url, **kwargs)
        if resp.status_code >= 200 and resp.status_code < 300:
            customers = []
            for customer in resp.body:
                customers.append(DirectDebitCustomer(**customer))
            return customers
        else:
            raise XenditError(resp)

    @staticmethod
    def initialize_tokenization(
        *,
        customer_id,
        channel_code,
        properties=None,
        metadata=None,
        x_idempotency_key=None,
        for_user_id=None,
        x_api_version=None,
        **kwargs,
    ):
        """Initialize Authorization Process and Account Token Creation
        (API Reference: Direct Debit/Initialize Linked Account Tokenization)

        Args:
          - customer_id (str)
          - channel_code (str)
          - **properties (DirectDebitCardLinking.Query or DirectDebitOnlineBanking.Query)
          - **metadata (dict)
          - **x_idempotency_key (str)
          - **for_user_id (str)
          - **x_api_version (str)

        Returns:
          DirectDebitToken

        Raises:
          XenditError

        """
        url = "/linked_account_tokens/auth"
        headers, body = _extract_params(
            locals(),
            func_object=DirectDebit.initialize_tokenization,
            headers_params=["for_user_id", "x_idempotency_key", "x_api_version"],
        )
        kwargs["headers"] = headers
        kwargs["body"] = body

        resp = _APIRequestor.post(url, **kwargs)
        if resp.status_code >= 200 and resp.status_code < 300:
            return DirectDebitToken(**resp.body)
        else:
            raise XenditError(resp)

    @staticmethod
    def validate_token_otp(
        *,
        linked_account_token_id,
        otp_code,
        x_idempotency_key=None,
        for_user_id=None,
        x_api_version=None,
        **kwargs,
    ):
        """Validates the Linked Account Token OTP
        (API Reference: Direct Debit/Validate OTP for Linked Account Token)

        Args:
          - linked_account_token_id (str)
          - otp_code (str)
          - **x_idempotency_key (str)
          - **for_user_id (str)
          - **x_api_version (str)

        Returns:
          DirectDebitToken

        Raises:
          XenditError

        """
        url = f"/linked_account_tokens/{linked_account_token_id}/validate_otp"
        headers, body = _extract_params(
            locals(),
            func_object=DirectDebit.validate_token_otp,
            headers_params=["for_user_id", "x_idempotency_key", "x_api_version"],
            ignore_params=["linked_account_token_id"],
        )
        kwargs["headers"] = headers
        kwargs["body"] = body

        resp = _APIRequestor.post(url, **kwargs)
        if resp.status_code >= 200 and resp.status_code < 300:
            return DirectDebitToken(**resp.body)
        else:
            raise XenditError(resp)

    @staticmethod
    def get_accessible_accounts_by_token(
        *, linked_account_token_id, for_user_id=None, x_api_version=None, **kwargs,
    ):
        """Get list of bank accounts accessible by the linked account token
        (API Reference: Direct Debit/Retrieve Accessible Accounts by Linked Account Token)

        Args:
          - linked_account_token_id (str)
          - **for_user_id (str)
          - **x_api_version (str)

        Returns:
          DirectDebitAccessibleAccount[]

        Raises:
          XenditError

        """
        url = f"/linked_account_tokens/{linked_account_token_id}/accounts"
        headers, _ = _extract_params(
            locals(),
            func_object=DirectDebit.get_accessible_accounts_by_token,
            headers_params=["for_user_id", "x_api_version"],
            ignore_params=["linked_account_token_id"],
        )
        kwargs["headers"] = headers

        resp = _APIRequestor.get(url, **kwargs)
        if resp.status_code >= 200 and resp.status_code < 300:
            accessible_accounts = []
            for accessible_account in resp.body:
                accessible_accounts.append(
                    DirectDebitAccessibleAccount(**accessible_account)
                )
            return accessible_accounts
        else:
            raise XenditError(resp)

    @staticmethod
    def create_payment_method(
        *,
        customer_id,
        type,
        properties,
        metadata=None,
        x_idempotency_key=None,
        for_user_id=None,
        x_api_version=None,
        **kwargs,
    ):
        """Create payment method for making payment
        (API Reference: Direct Debit/Create Payment Method)

        Args:
          - customer_id (str)
          - type (DirectDebitPaymentMethodType)
          - properties (DirectDebitPaymentMethodProperties.Query)
          - **metadata (dict)
          - **x_idempotency_key (str)
          - **for_user_id (str)
          - **x_api_version (str)

        Returns:
          DirectDebitPaymentMethod

        Raises:
          XenditError

        """
        type = DirectDebit._parse_payment_method_type(type)
        url = "/payment_methods"
        headers, body = _extract_params(
            locals(),
            func_object=DirectDebit.create_payment_method,
            headers_params=["for_user_id", "x_idempotency_key", "x_api_version"],
        )
        kwargs["headers"] = headers
        kwargs["body"] = body

        resp = _APIRequestor.post(url, **kwargs)
        if resp.status_code >= 200 and resp.status_code < 300:
            return DirectDebitPaymentMethod(**resp.body)
        else:
            raise XenditError(resp)

    @staticmethod
    def get_payment_methods_by_customer_id(
        *, customer_id, for_user_id=None, x_api_version=None, **kwargs,
    ):
        """Get payment methods by Customer ID
        (API Reference: Direct Debit/Get Payment Methods by Customer ID)

        Args:
          - customer_id (str)
          - **for_user_id (str)
          - **x_api_version (str)

        Returns:
          DirectDebitPaymentMethod[]

        Raises:
          XenditError

        """
        url = f"/payment_methods?customer_id={customer_id}"
        headers, body = _extract_params(
            locals(),
            func_object=DirectDebit.get_payment_methods_by_customer_id,
            headers_params=["for_user_id", "x_api_version"],
            ignore_params=["customer_id"],
        )
        kwargs["headers"] = headers

        resp = _APIRequestor.get(url, **kwargs)
        if resp.status_code >= 200 and resp.status_code < 300:
            payment_methods = []
            for payment_method in resp.body:
                payment_methods.append(DirectDebitPaymentMethod(**payment_method))
            return payment_methods
        else:
            raise XenditError(resp)

    @staticmethod
    def create_payment(
        *,
        reference_id,
        payment_method_id,
        currency,
        amount,
        callback_url,
        idempotency_key,
        enable_otp=None,
        description=None,
        basket=None,
        metadata=None,
        for_user_id=None,
        x_api_version=None,
        **kwargs,
    ):
        """Create a debit to pull funds from the end customer's account
        (API Reference: Direct Debit/Create Direct Debit Payment)

        Args:
          - reference_id (str)
          - payment_method_id (str)
          - currency (str)
          - amount (int)
          - callback_url (str)
          - idempotency_key (str)
          - **enable_otp (bool)
          - **description (str)
          - **basket (DirectDebitBasket[])
          - **metadata (dict)
          - **for_user_id (str)
          - **x_api_version (str)

        Returns:
          DirectDebitPayment

        Raises:
          XenditError

        """
        url = "/direct_debits"
        headers, body = _extract_params(
            locals(),
            func_object=DirectDebit.create_payment,
            headers_params=["for_user_id", "idempotency_key", "x_api_version"],
        )
        kwargs["headers"] = headers
        kwargs["body"] = body

        resp = _APIRequestor.post(url, **kwargs)
        if resp.status_code >= 200 and resp.status_code < 300:
            return DirectDebitPayment(**resp.body)
        else:
            raise XenditError(resp)

    @staticmethod
    def validate_payment_otp(
        *,
        direct_debit_id,
        otp_code,
        x_idempotency_key=None,
        for_user_id=None,
        x_api_version=None,
        **kwargs,
    ):
        """Validate OTP provided by end customer
        (API Reference: Direct Debit/Validate OTP for Direct Debit Payment)

        Args:
          - direct_debit_id (str)
          - otp_code (str)
          - **x_idempotency_key (str)
          - **for_user_id (str)
          - **x_api_version (str)

        Returns:
          DirectDebitPayment

        Raises:
          XenditError

        """
        url = f"/direct_debits/{direct_debit_id}/validate_otp/"
        headers, body = _extract_params(
            locals(),
            func_object=DirectDebit.validate_payment_otp,
            headers_params=["for_user_id", "x_idempotency_key", "x_api_version"],
            ignore_params=["direct_debit_id"],
        )
        kwargs["headers"] = headers
        kwargs["body"] = body

        resp = _APIRequestor.post(url, **kwargs)
        if resp.status_code >= 200 and resp.status_code < 300:
            return DirectDebitPayment(**resp.body)
        else:
            raise XenditError(resp)

    @staticmethod
    def get_payment_status(
        *, direct_debit_id, for_user_id=None, x_api_version=None, **kwargs,
    ):
        """Get the details of a direct debit payment
        (API Reference: Direct Debit/Get Direct Debit Payment Status by ID)

        Args:
          - direct_debit_id (str)
          - **for_user_id (str)
          - **x_api_version (str)

        Returns:
          DirectDebitPayment

        Raises:
          XenditError

        """
        url = f"/direct_debits/{direct_debit_id}/"
        headers, _ = _extract_params(
            locals(),
            func_object=DirectDebit.get_payment_status,
            headers_params=["for_user_id", "x_api_version"],
            ignore_params=["direct_debit_id"],
        )
        kwargs["headers"] = headers

        resp = _APIRequestor.get(url, **kwargs)
        if resp.status_code >= 200 and resp.status_code < 300:
            return DirectDebitPayment(**resp.body)
        else:
            raise XenditError(resp)

    @staticmethod
    def get_payment_status_by_ref_id(
        *, reference_id, for_user_id=None, x_api_version=None, **kwargs,
    ):
        """Get the details of a direct debit payment
        (API Reference: Direct Debit/Get Direct Debit Payment Status by Reference ID)

        Args:
          - reference_id (str)
          - **for_user_id (str)
          - **x_api_version (str)

        Returns:
          DirectDebitPayment[]

        Raises:
          XenditError

        """
        url = f"/direct_debits?reference_id={reference_id}"
        headers, _ = _extract_params(
            locals(),
            func_object=DirectDebit.get_payment_status,
            headers_params=["for_user_id", "x_api_version"],
            ignore_params=["reference_id"],
        )
        kwargs["headers"] = headers

        resp = _APIRequestor.get(url, **kwargs)
        if resp.status_code >= 200 and resp.status_code < 300:
            payments = []
            for payment in resp.body:
                payments.append(DirectDebitPayment(**payment))
            return payments
        else:
            raise XenditError(resp)

    @staticmethod
    def _parse_payment_method_type(payment_method_type):
        try:
            return payment_method_type.name
        except AttributeError:
            return payment_method_type



from xendit.models._base_model import BaseModel


class InvoiceBank(BaseModel):
    """Bank data detail in Invoice (API Reference: Invoice)

    Attributes:
      - bank_code (str)
      - collection_type (str)
      - bank_account_number (str)
      - transfer_amount (int)
      - bank_branch (str)
      - account_holder_name (str)
      - identity_amount (str)
    """

    bank_code: str
    collection_type: str
    bank_account_number: str
    transfer_amount: int
    bank_branch: str
    account_holder_name: str
    identity_amount: str



from xendit.models._base_model import BaseModel


class VirtualAccountPayment(BaseModel):
    """Payment class for Virtual Account (API Reference: Virtual Account)

    Attributes:
      - id (str)
      - payment_id (str)
      - callback_virtual_account_id (str)
      - external_id (str)
      - merchant_code (str)
      - account_number (str)
      - bank_code (str)
      - amount (int)
      - transaction_timestamp (str)

    Optional Attributes:
      - sender_name (str)
    """

    id: str
    payment_id: str
    callback_virtual_account_id: str
    external_id: str
    merchant_code: str
    account_number: str
    bank_code: str
    amount: int
    transaction_timestamp: str

    # Optional
    sender_name: str



from typing import Union

from xendit.models._base_model import BaseModel

from xendit.models.directdebit.token.direct_debit_card_link import DirectDebitCardLink
from xendit.models.directdebit.token.direct_debit_online_banking_link import (
    DirectDebitOnlineBankingLink,
)


class DirectDebitPaymentMethod(BaseModel):
    """Payment Method class in Direct Debit (API Reference: Direct Debit)

    Attributes:
      - id (str)
      - type (str)
      - properties (DirectDebitCardLink or DirectDebitOnlineBankingLink)
      - customer_id (str)
      - status (str)
      - created (str)
      - updated (str)
      - metadata (dict)

    """

    id: str
    type: str
    properties: Union[DirectDebitCardLink, DirectDebitOnlineBankingLink]
    customer_id: str
    status: str
    created: str
    updated: str
    metadata: dict



from xendit.models._base_model import BaseModel
from .balance_account_type import BalanceAccountType

from xendit.xendit_error import XenditError
from xendit._api_requestor import _APIRequestor
from xendit._extract_params import _extract_params


class Balance(BaseModel):
    """Balance class (API Reference: Balance)

    Related Classes:
      - Balance.AccountType

    Static Methods:
      - Balance.get (API Reference: /Get Balance)

    Attributes:
      - balance (int)
    """

    balance: int

    @staticmethod
    def get(
        *,
        account_type=BalanceAccountType.CASH,
        for_user_id=None,
        x_api_version=None,
        **kwargs,
    ):
        """Send GET request to retrieve balance (API Reference: Balance/Get Balance)

        Args:
          - account_type (BalanceAccountType)
          - **for_user_id (str) (XenPlatform only)
          - **x_api_version (str): API Version that will be used. If not provided will default to the latest

        Returns
          Balance

        Raises
          XenditError
        """
        headers, _ = _extract_params(
            locals(),
            func_object=Balance.get,
            headers_params=["for_user_id", "x_api_version"],
        )
        kwargs["headers"] = headers
        account_type = Balance._parse_value(account_type)
        url = f"/balance?account_type={account_type}"

        resp = _APIRequestor.get(url, **kwargs)
        if resp.status_code >= 200 and resp.status_code < 300:
            return Balance(**resp.body)
        else:
            raise XenditError(resp)

    @staticmethod
    def _parse_value(account_type):
        try:
            return account_type.name
        except AttributeError:
            return account_type



from xendit.models._base_model import BaseModel


class DANAPaymentStatus(BaseModel):
    """Payment Status for DANA (API Reference: eWallets)

    Attributes:
      - external_id (str)
      - business_id (str)
      - amount (int)
      - expiration_date (str)
      - checkout_url (str)
      - status (str)
    """

    external_id: str
    business_id: str
    amount: int
    expiration_date: str
    checkout_url: str
    status: str



class BaseQuery(dict):
    """Abstract class for query with object type"""

    def __init__(self, **kwargs):
        dict_params = {}
        for key, value in kwargs.items():
            if value is not None:
                dict_params[key] = value
        dict.__init__(self, dict_params)



from typing import List

from .disbursement_bank import DisbursementBank

from xendit.models._base_model import BaseModel

from xendit._api_requestor import _APIRequestor
from xendit._extract_params import _extract_params

from xendit.xendit_error import XenditError


class Disbursement(BaseModel):
    """Disbursement class (API Reference: Disbursement)

    Related Classes:
      - DisbursementBank

    Static Methods:
      - Disbursement.create (API Reference: /Create Disbursement)
      - Disbursement.get (API Reference: /Get Disbursement by ID)
      - Disbursement.get_by_ext_id (API Reference: /Get Disbursement by External ID)
      - Disbursement.get_available_banks (API Reference: /Get Available Banks)

    Attributes:
      - user_id (str)
      - external_id (str)
      - amount (int)
      - bank_code (str)
      - account_holder_name (str)
      - disbursement_description (str)
      - status (str)
      - id (str)

    Optional Attributes:
      - email_to (str[])
      - email_cc (str[])
      - email_bcc (str[])

    """

    user_id: str
    external_id: str
    amount: int
    bank_code: str
    account_holder_name: str
    disbursement_description: str
    status: str
    id: str

    # Optional
    email_to: List[str]
    email_cc: List[str]
    email_bcc: List[str]

    @staticmethod
    def create(
        *,
        external_id,
        bank_code,
        account_holder_name,
        account_number,
        description,
        amount,
        email_to=[],
        email_cc=[],
        email_bcc=[],
        x_idempotency_key=None,
        for_user_id=None,
        x_api_version=None,
        **kwargs,
    ):
        """Send POST Request to create Disbursement (API Reference: Disbursement/Create Disbursement)

        Args:
          - external_id (str)
          - bank_code (str)
          - account_holder_name (str)
          - account_number (str)
          - description (str)
          - amount (int)
          - **email_to (str[])
          - **email_cc (str[])
          - **email_bcc (str[])
          - **for_user_id (str)
          - **x_idempotency_key (str)
          - **x_api_version (str): API Version that will be used. If not provided will default to the latest

        Returns:
          Disbursement

        Raises:
          XenditError

        """
        url = "/disbursements"
        headers, body = _extract_params(
            locals(),
            func_object=Disbursement.create,
            headers_params=["for_user_id", "x_idempotency_key", "x_api_version"],
        )
        kwargs["headers"] = headers
        kwargs["body"] = body

        resp = _APIRequestor.post(url, **kwargs)
        if resp.status_code >= 200 and resp.status_code < 300:
            return Disbursement(**resp.body)
        else:
            raise XenditError(resp)

    @staticmethod
    def get(*, id, for_user_id=None, x_api_version=None, **kwargs):
        """Get Disbursement detail by ID (API Reference: Disbursement/Get Disbursement by ID)

        Args:
          - id (str)
          - **for_user_id (str)
          - **x_api_version (str): API Version that will be used. If not provided will default to the latest

        Returns:
          Disbursement

        Raises:
          XenditError

        """
        url = f"/disbursements/{id}"
        headers, _ = _extract_params(
            locals(),
            func_object=Disbursement.get,
            headers_params=["for_user_id", "x_api_version"],
            ignore_params=["id"],
        )
        kwargs["headers"] = headers

        resp = _APIRequestor.get(url, **kwargs)
        if resp.status_code >= 200 and resp.status_code < 300:
            return Disbursement(**resp.body)
        else:
            raise XenditError(resp)

    @staticmethod
    def get_by_ext_id(*, external_id, for_user_id=None, x_api_version=None, **kwargs):
        """Get Disbursement detail by external ID (API Reference: Disbursement/Get Disbursement by External ID)

        Args:
          - external_id (str)
          - **for_user_id (str)
          - **x_api_version (str): API Version that will be used. If not provided will default to the latest

        Returns:
          Disbursement

        Raises:
          XenditError

        """
        url = f"/disbursements?external_id={external_id}"
        headers, _ = _extract_params(
            locals(),
            func_object=Disbursement.get_by_ext_id,
            headers_params=["for_user_id", "x_api_version"],
            ignore_params=["external_id"],
        )
        kwargs["headers"] = headers

        resp = _APIRequestor.get(url, **kwargs)
        if resp.status_code >= 200 and resp.status_code < 300:
            disbursements = []
            for disbursement in resp.body:
                disbursements.append(Disbursement(**disbursement))
            return disbursements
        else:
            raise XenditError(resp)

    @staticmethod
    def get_available_banks(*, for_user_id=None, x_api_version=None, **kwargs):
        """Get Available Banks (API Reference: Disbursement/Get Available Banks)

        Args:
          - **for_user_id (str)
          - **x_api_version (str): API Version that will be used. If not provided will default to the latest

        Returns:
          List of DisbursementBank

        Raises:
          XenditError

        """
        url = "/available_disbursements_banks"
        headers, _ = _extract_params(
            locals(),
            func_object=Disbursement.get_available_banks,
            headers_params=["for_user_id", "x_api_version"],
        )
        kwargs["headers"] = headers

        resp = _APIRequestor.get(url, **kwargs)
        if resp.status_code >= 200 and resp.status_code < 300:
            disbursement_banks = []
            for bank in resp.body:
                disbursement_banks.append(DisbursementBank(**bank))
            return disbursement_banks
        else:
            raise XenditError(resp)



from .charge import (
    CreditCardCharge,
    CreditCardChargeAddress,
    CreditCardChargeBillingDetails,
    CreditCardChargeInstallment,
    CreditCardChargePromotion,
)

from .credit_card_promotion import CreditCardPromotion
from .credit_card_refund import CreditCardRefund
from .credit_card_reverse_authorization import CreditCardReverseAuthorization

from xendit.models._base_model import BaseModel

from xendit._api_requestor import _APIRequestor
from xendit._extract_params import _extract_params

from xendit.xendit_error import XenditError


class CreditCard(BaseModel):
    """CreditCard class (API Reference: CreditCard)

    Related Classes:
      - CreditCardCharge
      - CreditCardChargeAddress
      - CreditCardChargeBillingDetails
      - CreditCardChargeInstallment
      - CreditCardChargePromotion

    Static Methods:
      - CreditCard.create_authorization (API Reference: /Create Authorization)
      - CreditCard.reverse_authorization (API Reference: /Reverse Authorization)
      - CreditCard.create_charge (API Reference: /Create Charge)
      - CreditCard.capture_charge (API Reference: /Capture Charge)
      - CreditCard.get_charge (API Reference: /Get Charge)
      - CreditCard.create_refund (API Reference: /Create Refund)
      - CreditCard.create_promotion (API Reference: /Create Promotion)

    Static Methods for Object Creation:
      - CreditCard.helper_create_installment (For Installment in create_authorization and create_charge)
      - CreditCard.helper_create_charge_promotion (For Promotion in create_authorization and create_charge)
      - CreditCard.helper_create_billing_details (For BillingDetails in create_authorization and create_charge)
      - CreditCard.helper_create_address (For Address in create_billing_details)
    """

    @staticmethod
    def helper_create_installment(*, count=None, interval=None, **kwargs):
        """Construct Installments Object for Charge

        Args:
          - **count (int)
          - **interval (str)

        Return:
          - CreditCardChargeInstallment.Query
        """
        params = locals()
        del params["kwargs"]

        return CreditCardChargeInstallment.Query(**params)

    @staticmethod
    def helper_create_charge_promotion(
        *, reference_id=None, original_amount=None, **kwargs
    ):
        """Construct Promotion Object for Charge

        Args:
          - **reference_id (str)
          - **original_amount (float)

        Return:
          - CreditCardChargePromotion.Query
        """
        params = locals()
        del params["kwargs"]

        return CreditCardChargePromotion.Query(**params)

    @staticmethod
    def helper_create_billing_details(
        *,
        given_names,
        address,
        middle_name=None,
        surname=None,
        email=None,
        mobile_number=None,
        phone_number=None,
        **kwargs,
    ):
        """Construct Billing Details Object for Charge

        Args:
          - given_names (str)
          - address (CreditCardCharge.Address)
          - **middle_name (str)
          - **surname (str)
          - **email (str)
          - **mobile_number (str)
          - **phone_number (str)

        Return:
          - CreditCardChargeBillingDetails.Query
        """
        params = locals()
        del params["kwargs"]

        return CreditCardChargeBillingDetails.Query(**params)

    @staticmethod
    def helper_create_address(
        *,
        country,
        street_line1=None,
        street_line2=None,
        city=None,
        province=None,
        state=None,
        postal_code=None,
        description=None,
        **kwargs,
    ):
        """Construct Address Object for Billing Details

        Args:
          - country (str)
          - **street_line1 (str)
          - **street_line2 (str)
          - **city (str)
          - **province (str)
          - **state (str)
          - **postal_code (str)
          - **description (str)

        Return:
          - CreditCardChargeAddress.Query
        """
        params = locals()
        del params["kwargs"]

        return CreditCardChargeAddress.Query(**params)

    @staticmethod
    def create_authorization(
        *,
        token_id,
        external_id,
        amount,
        capture=False,
        authentication_id=None,
        card_cvn=None,
        descriptor=None,
        currency=None,
        mid_label=None,
        billing_details=None,
        promotion=None,
        installment=None,
        x_idempotency_key=None,
        for_user_id=None,
        x_api_version=None,
        **kwargs,
    ):
        """Send POST Request to create Credit Card Authorization (API Reference: Credit Card/Create Authorization)

        Args:
          - token_id (str)
          - external_id (str)
          - amount (float)
          - authentication_id (str)
          - card_cvn (str)
          - **capture (bool)
          - **descriptor (str)
          - **currency (str)
          - **mid_label (str)
          - **billing_details (CreditCardCharge.BillingDetails.Query)
          - **promotion (CreditCardCharge.Promotion.Query)
          - **installment (CreditCardCharge.Installment.Query)
          - **x_idempotency_key (str)
          - **for_user_id (str)
          - **x_api_version (str)


        Returns:
          CreditCardCharge

        Raises:
          XenditError

        """
        params = locals()
        params = {**params, **kwargs}
        del params["kwargs"]

        return CreditCard.create_charge(**params)

    @staticmethod
    def reverse_authorization(
        *,
        credit_card_charge_id,
        external_id,
        x_idempotency_key=None,
        for_user_id=None,
        x_api_version=None,
        **kwargs,
    ):
        """Send POST Request to reverse Credit Card Authorization (API Reference: Credit Card/Reverse Authorization)

        Args:
          - credit_card_charge_id (str)
          - external_id (str)
          - **x_idempotency_key (str)
          - **for_user_id (str)
          - **x_api_version

        Returns:
          CreditCardReverseAuthorization

        Raises:
          XenditError

        """
        url = f"/credit_card_charges/{credit_card_charge_id}/auth_reversal"
        headers, body = _extract_params(
            locals(),
            func_object=CreditCard.reverse_authorization,
            headers_params=["for_user_id", "x_idempotency_key", "x_api_version"],
            ignore_params=["credit_card_charge_id"],
        )
        kwargs["headers"] = headers
        kwargs["body"] = body

        resp = _APIRequestor.post(url, **kwargs)
        if resp.status_code >= 200 and resp.status_code < 300:
            return CreditCardReverseAuthorization(**resp.body)
        else:
            raise XenditError(resp)

    @staticmethod
    def create_charge(
        *,
        token_id,
        external_id,
        amount,
        capture=True,
        authentication_id=None,
        card_cvn=None,
        descriptor=None,
        currency=None,
        mid_label=None,
        billing_details=None,
        promotion=None,
        installment=None,
        x_idempotency_key=None,
        for_user_id=None,
        x_api_version=None,
        **kwargs,
    ):
        """Send POST Request to create Credit Card Charge (API Reference: Credit Card/Create Charge)

        Args:
          - token_id (str)
          - external_id (str)
          - amount (float)
          - authentication_id (str)
          - card_cvn (str)
          - **capture (bool)
          - **descriptor (str)
          - **currency (str)
          - **mid_label (str)
          - **billing_details (CreditCardCharge.BillingDetails.Query)
          - **promotion (CreditCardCharge.Promotion.Query)
          - **installment (CreditCardCharge.Installment.Query)
          - **x_idempotency_key (str)
          - **for_user_id (str)
          - **x_api_version (str)

        Returns:
          CreditCardCharge

        Raises:
          XenditError

        """
        url = "/credit_card_charges"
        headers, body = _extract_params(
            locals(),
            func_object=CreditCard.create_charge,
            headers_params=["for_user_id", "x_idempotency_key", "x_api_version"],
        )
        kwargs["headers"] = headers
        kwargs["body"] = body

        resp = _APIRequestor.post(url, **kwargs)
        if resp.status_code >= 200 and resp.status_code < 300:
            return CreditCardCharge(**resp.body)
        else:
            raise XenditError(resp)

    @staticmethod
    def capture_charge(
        *,
        credit_card_charge_id,
        amount,
        x_idempotency_key=None,
        for_user_id=None,
        x_api_version=None,
        **kwargs,
    ):
        """Send POST Request to capture Credit Card Charge (API Reference: Credit Card/Capture Charge)

        Args:
          - credit_card_charge_id (str)
          - amount (str)
          - **x_idempotency_key (str)
          - **for_user_id (str)
          - **x_api_version (str)

        Returns:
          CreditCardCharge

        Raises:
          XenditError

        """
        url = f"/credit_card_charges/{credit_card_charge_id}/capture"
        headers, body = _extract_params(
            locals(),
            func_object=CreditCard.capture_charge,
            headers_params=["for_user_id", "x_idempotency_key", "x_api_version"],
            ignore_params=["credit_card_charge_id"],
        )
        kwargs["headers"] = headers
        kwargs["body"] = body

        resp = _APIRequestor.post(url, **kwargs)
        if resp.status_code >= 200 and resp.status_code < 300:
            return CreditCardCharge(**resp.body)
        else:
            raise XenditError(resp)

    @staticmethod
    def get_charge(
        *, credit_card_charge_id, for_user_id=None, x_api_version=None, **kwargs,
    ):
        """Get Detail of Credit Card Charge (API Reference: Credit Card/Get Charge)

        Args:
          - credit_card_charge_id (str)
          - **for_user_id (str)
          - **x_api_version (str)

        Returns:
          CreditCardCharge

        Raises:
          XenditError

        """
        url = f"/credit_card_charges/{credit_card_charge_id}"
        headers, _ = _extract_params(
            locals(),
            func_object=CreditCard.get_charge,
            headers_params=["for_user_id", "x_idempotency_key", "x_api_version"],
            ignore_params=["credit_card_charge_id"],
        )
        kwargs["headers"] = headers

        resp = _APIRequestor.get(url, **kwargs)
        if resp.status_code >= 200 and resp.status_code < 300:
            return CreditCardCharge(**resp.body)
        else:
            raise XenditError(resp)

    @staticmethod
    def create_refund(
        *,
        credit_card_charge_id,
        amount,
        external_id,
        x_idempotency_key=None,
        for_user_id=None,
        x_api_version=None,
        **kwargs,
    ):
        """Send POST Request to create refund for Credit Card (API Reference: Credit Card/Create Refund)

        Args:
          - credit_card_charge_id (str)
          - amount (str)
          - external_id (str)
          - **x_idempotency_key (str)
          - **for_user_id (str)
          - **x_api_version (str)

        Returns:
          CreditCardRefund

        Raises:
          XenditError

        """
        url = f"/credit_card_charges/{credit_card_charge_id}/refunds"
        headers, body = _extract_params(
            locals(),
            func_object=CreditCard.create_refund,
            headers_params=["for_user_id", "x_idempotency_key", "x_api_version"],
            ignore_params=["credit_card_charge_id"],
        )
        kwargs["headers"] = headers
        kwargs["body"] = body

        resp = _APIRequestor.post(url, **kwargs)
        if resp.status_code >= 200 and resp.status_code < 300:
            return CreditCardRefund(**resp.body)
        else:
            raise XenditError(resp)

    @staticmethod
    def create_promotion(
        *,
        reference_id,
        description,
        start_time,
        end_time,
        promo_code=None,
        bin_list=None,
        channel_code=None,
        discount_percent=None,
        discount_amount=None,
        currency=None,
        x_idempotency_key=None,
        for_user_id=None,
        x_api_version=None,
        **kwargs,
    ):
        """Send POST Request to create new Promotion (API Reference: Credit Card/Create Promotion)

        Args:
          - reference_id (str)
          - description (str)
          - start_time (str)
          - end_time (str)
          - **promo_code (str)
          - **bin_list (str[])
          - **channel_code (str)
          - **discount_percent (float)
          - **discount_amount (float)
          - **currency (str)
          - **x_idempotency_key (str)
          - **for_user_id (str)
          - **x_api_version (str)

        Returns:
          CreditCardPromotion

        Raises:
          XenditError

        """
        url = "/promotions"
        headers, body = _extract_params(
            locals(),
            func_object=CreditCard.create_promotion,
            headers_params=["for_user_id", "x_idempotency_key", "x_api_version"],
        )
        kwargs["headers"] = headers
        kwargs["body"] = body

        resp = _APIRequestor.post(url, **kwargs)
        if resp.status_code >= 200 and resp.status_code < 300:
            return CreditCardPromotion(**resp.body)
        else:
            raise XenditError(resp)



from xendit.models._base_model import BaseModel
from xendit.models._base_query import BaseQuery


class DirectDebitCardLink(BaseModel):
    """Card Linking class in Direct Debit Tokenization (API Reference: Direct Debit)

    Attributes:
      - card_last_four (str)
      - card_expiry (str)
      - currency (str)
      - description (str)

    """

    card_last_four: str
    card_expiry: str
    currency: str
    description: str

    class Query(BaseQuery):
        """Card Linking class in Direct Debit Tokenization Query (API Reference: Direct Debit)

        Initialize this for initialize_tokenization

        Attributes:
          - account_mobile_number (str)
          - card_last_four (str)
          - card_expiry (str)
          - account_email (str)
        """

        account_mobile_number: str
        card_last_four: str
        card_expiry: str
        account_email: str



from typing import List

from xendit.models._base_model import BaseModel

from xendit._api_requestor import _APIRequestor
from xendit._extract_params import _extract_params

from xendit.xendit_error import XenditError

from .invoice_bank import InvoiceBank
from .invoice_retail_outlet import InvoiceRetailOutlet


class Invoice(BaseModel):
    """Invoice class (API Reference: Invoice)

    Related Classes:
      - InvoiceBank

    Static Methods:
      - Invoice.create (API Reference: /Create Invoice)
      - Invoice.get (API Reference: /Get Invoice)
      - Invoice.expire (API Reference: /Expire Invoice)
      - Invoice.list_all (API Reference: /List All Invoice)

    Attributes:
      - id (str)
      - external_id (str)
      - user_id (str)
      - status (str)
      - merchant_name (str)
      - merchant_profile_picture_url (str)
      - amount (int)
      - payer_email (str)
      - description (str)
      - expiry_date (str)
      - invoice_url (str)
      - available_banks (InvoiceBank[])
      - available_ewallets (?)
      - should_exclude_credit_card (bool)
      - should_send_email (bool)
      - created (str)
      - updated (str)
      - currency (str)

    """

    id: str
    external_id: str
    user_id: str
    status: str
    merchant_name: str
    merchant_profile_picture_url: str
    amount: int
    payer_email: str
    description: str
    invoice_url: str
    expiry_date: str
    available_banks: List[InvoiceBank]
    available_ewallets: List[None]
    available_retail_outlets: List[InvoiceRetailOutlet]
    should_exclude_credit_card: bool
    should_send_email: bool
    created: str
    updated: str
    currency: str

    @staticmethod
    def create(
        *,
        external_id,
        payer_email,
        description,
        amount,
        should_send_email=None,
        callback_virtual_account_id=None,
        invoice_duration=None,
        success_redirect_url=None,
        failure_redirect_url=None,
        payment_methods=None,
        mid_label=None,
        currency=None,
        fixed_va=None,
        reminder_time=None,
        for_user_id=None,
        x_idempotency_key=None,
        x_api_version=None,
        **kwargs,
    ):
        """Send POST Request to create Invoice (API Reference: Invoice/Create Invoice)

        Args:
          - external_id (str)
          - payer_email (str)
          - description (str)
          - amount (int)
          - **should_send_email (bool)
          - **callback_virtual_account_id (str)
          - **invoice_duration (int)
          - **success_redirect_url (str)
          - **failure_redirect_url (str)
          - **payment_methods (str[])
          - **mid_label (str)
          - **currency (str)
          - **fixed_va (bool)
          - **reminder_time (int)
          - **for_user_id (str) (XenPlatforms only)
          - **x_idempotency_key (str)
          - **x_api_version (str): API Version that will be used. If not provided will default to the latest

        Returns:
          Invoice

        Raises:
          XenditError

        """
        url = "/v2/invoices"
        headers, body = _extract_params(
            locals(),
            func_object=Invoice.create,
            headers_params=["for_user_id", "x_idempotency_key", "x_api_version"],
        )
        kwargs["headers"] = headers
        kwargs["body"] = body

        resp = _APIRequestor.post(url, **kwargs)
        if resp.status_code >= 200 and resp.status_code < 300:
            return Invoice(**resp.body)
        else:
            raise XenditError(resp)

    @staticmethod
    def get(
        *, invoice_id, for_user_id=None, x_api_version=None, **kwargs,
    ):
        """Get Invoice (API Reference: Invoice/Get Invoice)

        Args:
          - invoice_id (str)
          - **for_user_id (str)
          - **x_api_version (str): API Version that will be used. If not provided will default to the latest

        Returns:
          Invoice

        Raises:
          XenditError

        """
        url = f"/v2/invoices/{invoice_id}"
        headers, _ = _extract_params(
            locals(),
            func_object=Invoice.get,
            headers_params=["for_user_id", "x_api_version"],
            ignore_params=["invoice_id"],
        )
        kwargs["headers"] = headers

        resp = _APIRequestor.get(url, **kwargs)
        if resp.status_code >= 200 and resp.status_code < 300:
            return Invoice(**resp.body)
        else:
            raise XenditError(resp)

    @staticmethod
    def expire(
        *,
        invoice_id,
        for_user_id=None,
        x_idempotency_key=None,
        x_api_version=None,
        **kwargs,
    ):
        """Cancel an already created invoice (API Reference: Invoice/Expire Invoice)

        Args:
          - invoice_id (str)
          - **for_user_id (str)
          - **x_idempotency_key (str)
          - **x_api_version (str): API Version that will be used. If not provided will default to the latest

        Returns:
          Invoice

        Raises:
          XenditError

        """
        url = f"/invoices/{invoice_id}/expire!"
        headers, body = _extract_params(
            locals(),
            func_object=Invoice.expire,
            headers_params=["for_user_id", "x_idempotency_key", "x_api_version"],
            ignore_params=["invoice_id"],
        )
        kwargs["headers"] = headers
        kwargs["body"] = body

        resp = _APIRequestor.post(url, **kwargs)
        if resp.status_code >= 200 and resp.status_code < 300:
            return Invoice(**resp.body)
        else:
            raise XenditError(resp)

    @staticmethod
    def list_all(
        *,
        statuses=None,
        limit=None,
        created_after=None,
        created_before=None,
        paid_after=None,
        paid_before=None,
        expired_after=None,
        expired_before=None,
        last_invoice_id=None,
        client_types=None,
        payment_channels=None,
        on_demand_link=None,
        recurring_payment_id=None,
        for_user_id=None,
        x_api_version=None,
        **kwargs,
    ):
        """List all Invoice based on filter (API Reference: Invoice/List All Invoice)

        Args:
          - statuses (str[])
          - limit (int)
          - created_after (str)
          - created_before (str)
          - paid_after (str)
          - paid_before (str)
          - expired_after (str)
          - expired_before (str)
          - last_invoice_id (str)
          - client_types (str[])
          - payment_channels (str[])
          - on_demand_link (str)
          - recurring_payment_id (str)
          - **for_user_id (str)
          - **x_api_version (str): API Version that will be used. If not provided will default to the latest

        Returns:
          Invoice

        Raises:
          XenditError

        """
        url = "/v2/invoices"
        headers, params = _extract_params(
            locals(),
            func_object=Invoice.list_all,
            headers_params=["for_user_id", "x_api_version"],
            ignore_params=[],
        )
        kwargs["headers"] = headers
        kwargs["params"] = params

        resp = _APIRequestor.get(url, **kwargs)
        if resp.status_code >= 200 and resp.status_code < 300:
            invoices = []
            for invoice in resp.body:
                invoices.append(Invoice(**invoice))
            return invoices
        else:
            raise XenditError(resp)



from xendit.models._base_model import BaseModel

from xendit._api_requestor import _APIRequestor
from xendit._extract_params import _extract_params

from xendit.xendit_error import XenditError


class RetailOutlet(BaseModel):
    """RetailOutlet class (API Reference: Retail Outlets)

    Static Methods:
      - RetailOutlet.create_fixed_payment_code (API Reference: /Create Fixed Payment Code)
      - RetailOutlet.update_fixed_payment_code (API Reference: /Update Fixed Payment Code)
      - RetailOutlet.get_fixed_payment_code (API Reference: /Get Fixed Payment Code)

    Attributes:
      - owner_id (str)
      - external_id (str)
      - retail_outlet_name (str)
      - prefix (str)
      - name (str)
      - payment_code (str)
      - expected_amount (int)
      - is_single_use (bool)
      - expiration_date (str) (ISO 8601 Date)
      - id (str)

    """

    owner_id: str
    external_id: str
    retail_outlet_name: str
    prefix: str
    name: str
    payment_code: str
    expected_amount: int
    is_single_use: bool
    expiration_Date: str
    id: str
    status: str
    type: str

    @staticmethod
    def create_fixed_payment_code(
        *,
        external_id,
        retail_outlet_name,
        name,
        expected_amount,
        payment_code=None,
        expiration_date=None,
        is_single_use=None,
        for_user_id=None,
        x_idempotency_key=None,
        x_api_version=None,
        **kwargs,
    ):
        """Send POST Request to create fixed payment code (API Reference: Retail Outlets/Create Fixed Payment Code)

        Args:
          - external_id (str)
          - retail_outlet_name (str)
          - name (str)
          - expected_amount (int)
          - **payment_code (str)
          - **expiration_date (str) (ISO 8601 Date)
          - **is_single_use (bool)
          - **for_user_id (str)
          - **x_idempotency_key (str)
          - **x_api_version (str): API Version that will be used. If not provided will default to the latest

        Returns:
          RetailOutlet

        Raises:
          XenditError

        """
        url = "/fixed_payment_code"
        headers, body = _extract_params(
            locals(),
            func_object=RetailOutlet.create_fixed_payment_code,
            headers_params=["for_user_id", "x_idempotency_key", "x_api_version"],
        )
        kwargs["headers"] = headers
        kwargs["body"] = body

        resp = _APIRequestor.post(url, **kwargs)
        if resp.status_code >= 200 and resp.status_code < 300:
            return RetailOutlet(**resp.body)
        else:
            raise XenditError(resp)

    @staticmethod
    def update_fixed_payment_code(
        *,
        fixed_payment_code_id,
        name=None,
        expected_amount=None,
        expiration_date=None,
        for_user_id=None,
        x_idempotency_key=None,
        x_api_version=None,
        **kwargs,
    ):
        """Update fixed payment code (API Reference: Retail Outlets/Update Fixed Payment Code)

        Args:
          - fixed_payment_code_id (str)
          - **name (str)
          - **expected_amount (int)
          - **expiration_date (str) (ISO 8601 Date)
          - **for_user_id (str)
          - **x_idempotency_key (str)
          - **x_api_version (str): API Version that will be used. If not provided will default to the latest

        Returns:
          RetailOutlet

        Raises:
          XenditError

        """
        url = f"/fixed_payment_code/{fixed_payment_code_id}"
        headers, body = _extract_params(
            locals(),
            func_object=RetailOutlet.update_fixed_payment_code,
            headers_params=["for_user_id", "x_idempotency_key", "x_api_version"],
        )
        kwargs["headers"] = headers
        kwargs["body"] = body

        resp = _APIRequestor.patch(url, **kwargs)
        if resp.status_code >= 200 and resp.status_code < 300:
            return RetailOutlet(**resp.body)
        else:
            raise XenditError(resp)

    @staticmethod
    def get_fixed_payment_code(
        *, fixed_payment_code_id, for_user_id=None, x_api_version=None, **kwargs
    ):
        """Get the detail of given Fixed Payment Code (API Reference: Retail Outlets/Get Fixed Payment Code)

        Args:
          - fixed_payment_code_id (str)
          - **for_user_id (str) (XenPlatforms only)
          - **x_api_version (str): API Version that will be used. If not provided will default to the latest

        Returns:
          RetailOutlet

        Raises:
          XenditError

        """
        url = f"/fixed_payment_code/{fixed_payment_code_id}"
        headers, _ = _extract_params(
            locals(),
            func_object=RetailOutlet.update_fixed_payment_code,
            headers_params=["for_user_id", "x_api_version"],
        )
        kwargs["headers"] = headers

        resp = _APIRequestor.get(url, **kwargs)
        if resp.status_code >= 200 and resp.status_code < 300:
            return RetailOutlet(**resp.body)
        else:
            raise XenditError(resp)



from xendit.models._base_model import BaseModel

from xendit._api_requestor import _APIRequestor
from xendit._extract_params import _extract_params

from xendit.xendit_error import XenditError


class RecurringPayment(BaseModel):
    """RecurringPayment class (API Reference: RecurringPayment)

    Static Methods:
      - RecurringPayment.create_recurring_payment (API Reference: /Create Recurring Payment)
      - RecurringPayment.get_recurring_payment (API Reference: /Get Recurring Payment)
      - RecurringPayment.edit_recurring_payment (API Reference: /Edit Recurring Payment)
      - RecurringPayment.stop_recurring_payment (API Reference: /Stop Recurring Payment)
      - RecurringPayment.pause_recurring_payment (API Reference: /Pause Recurring Payment)
      - RecurringPayment.resume_recurring_payment (API Reference: /Resume Recurring Payment)

    Static Methods for Object Creation:
      - RecurringPayment.helper_create_installment (For Installment in create_authorization and create_charge)

    Attributes:
      - id (str)
      - user_id (str)
      - external_id (str)
      - status (str)
      - amount (float)
      - payer_email (str)
      - description (str)
      - should_send_email (bool)
      - interval (str)
      - interval_count (int)
      - recurrence_progress (int)
      - last_created_invoice_url (str)
      - credit_card_token (str)
      - created (str)
      - updated (str)
      - recharge (bool)
      - payment_method_id (str)

    Optional Attributes:
      - success_redirect_url (str)
      - failure_redirect_url (str)
      - invoice_duration (int)
      - charge_immediately (bool)
      - currency (str)
    """

    id: str
    user_id: str
    external_id: str
    status: str
    amount: float
    payer_email: str
    description: str
    should_send_email: bool
    interval: str
    interval_count: int
    recurrence_progress: int
    last_created_invoice_url: str
    credit_card_token: str
    created: str
    updated: str
    recharge: bool
    payment_method_id: str

    # Optional
    success_redirect_url: str
    failure_redirect_url: str
    invoice_duration: int
    charge_immediately: bool
    currency: str

    @staticmethod
    def create_recurring_payment(
        *,
        external_id,
        payer_email,
        description,
        amount,
        interval,
        interval_count,
        total_recurrence=None,
        invoice_duration=None,
        should_send_email=None,
        missed_payment_action=None,
        credit_card_token=None,
        start_date=None,
        success_redirect_url=None,
        failure_redirect_url=None,
        recharge=None,
        charge_immediately=None,
        payment_method_id=None,
        currency=None,
        x_idempotency_key=None,
        for_user_id=None,
        x_api_version=None,
        **kwargs,
    ):
        """Send POST Request to create refund for Credit Card (API Reference: Credit Card/Create Refund)

        Args:
          - external_id (str)
          - payer_email (str)
          - description (str)
          - amount (float)
          - interval (str)
          - interval_count (int)
          - **total_recurrence (int)
          - **invoice_duration (int)
          - **should_send_email (int)
          - **missed_payment_action (str)
          - **credit_card_token (str)
          - **start_date (str)
          - **success_redirect_url (str)
          - **failure_redirect_url (str)
          - **recharge (bool)
          - **charge_immediately (bool)
          - **payment_method_id (str)
          - **currency (str)
          - **x_idempotency_key (str)
          - **for_user_id (str)
          - **x_api_version (str)

        Returns:
          RecurringPayment

        Raises:
          XenditError

        """
        url = "/recurring_payments"
        headers, body = _extract_params(
            locals(),
            func_object=RecurringPayment.create_recurring_payment,
            headers_params=["for_user_id", "x_idempotency_key", "x_api_version"],
        )
        kwargs["headers"] = headers
        kwargs["body"] = body

        resp = _APIRequestor.post(url, **kwargs)
        if resp.status_code >= 200 and resp.status_code < 300:
            return RecurringPayment(**resp.body)
        else:
            raise XenditError(resp)

    @staticmethod
    def get_recurring_payment(
        *, id, for_user_id=None, x_api_version=None, **kwargs,
    ):
        """Get Recurring Payment by ID (API Reference: Recurring Payment/Get Recurring Payment)

        Args:
          - id (str)
          - **for_user_id (str)
          - **x_api_version (str)

        Returns:
          RecurringPayment

        Raises:
          XenditError

        """
        url = f"/recurring_payments/{id}"
        headers, _ = _extract_params(
            locals(),
            func_object=RecurringPayment.get_recurring_payment,
            headers_params=["for_user_id", "x_api_version"],
        )
        kwargs["headers"] = headers

        resp = _APIRequestor.get(url, **kwargs)
        if resp.status_code >= 200 and resp.status_code < 300:
            return RecurringPayment(**resp.body)
        else:
            raise XenditError(resp)

    @staticmethod
    def edit_recurring_payment(
        *,
        id,
        amount=None,
        credit_card_token=None,
        interval=None,
        interval_count=None,
        should_send_email=None,
        invoice_duration=None,
        missed_payment_action=None,
        payment_method_id=None,
        customer_id=None,
        x_idempotency_key=None,
        for_user_id=None,
        x_api_version=None,
        **kwargs,
    ):
        """Edit Recurring Payment Data (API Reference: Recurring Payment/Edit Recurring Payment)

        Args:
          - id (str)
          - **amount (int)
          - **credit_card_token (str)
          - **interval (str)
          - **interval_count (int)
          - **should_send_email (bool)
          - **invoice_duration (int)
          - **missed_payment_action (str)
          - **payment_method_id (str)
          - **customer_id (str)
          - **x_idempotency_key (str)
          - **for_user_id (str)
          - **x_api_version (str)

        Returns:
          RecurringPayment

        Raises:
          XenditError

        """
        url = f"/recurring_payments/{id}"
        headers, body = _extract_params(
            locals(),
            func_object=RecurringPayment.edit_recurring_payment,
            headers_params=["for_user_id", "x_idempotency_key", "x_api_version"],
            ignore_params=["id"],
        )
        kwargs["headers"] = headers
        kwargs["body"] = body

        resp = _APIRequestor.patch(url, **kwargs)
        if resp.status_code >= 200 and resp.status_code < 300:
            return RecurringPayment(**resp.body)
        else:
            raise XenditError(resp)

    @staticmethod
    def stop_recurring_payment(
        *, id, x_idempotency_key=None, for_user_id=None, x_api_version=None, **kwargs,
    ):
        """Stop Recurring Payment (API Reference: Recurring Payment/Stop Recurring Payment)

        Args:
          - id (str)
          - **x_idempotency_key (str)
          - **for_user_id (str)
          - **x_api_version (str)

        Returns:
          RecurringPayment

        Raises:
          XenditError

        """
        url = f"/recurring_payments/{id}/stop!"
        headers, body = _extract_params(
            locals(),
            func_object=RecurringPayment.stop_recurring_payment,
            headers_params=["for_user_id", "x_idempotency_key", "x_api_version"],
            ignore_params=["id"],
        )
        kwargs["headers"] = headers
        kwargs["body"] = body

        resp = _APIRequestor.post(url, **kwargs)
        if resp.status_code >= 200 and resp.status_code < 300:
            return RecurringPayment(**resp.body)
        else:
            raise XenditError(resp)

    @staticmethod
    def pause_recurring_payment(
        *, id, x_idempotency_key=None, for_user_id=None, x_api_version=None, **kwargs,
    ):
        """Pause Recurring Payment (API Reference: Recurring Payment/Pause Recurring Payment)

        Args:
          - id (str)
          - **x_idempotency_key (str)
          - **for_user_id (str)
          - **x_api_version (str)

        Returns:
          RecurringPayment

        Raises:
          XenditError

        """
        url = f"/recurring_payments/{id}/pause!"
        headers, body = _extract_params(
            locals(),
            func_object=RecurringPayment.stop_recurring_payment,
            headers_params=["for_user_id", "x_idempotency_key", "x_api_version"],
            ignore_params=["id"],
        )
        kwargs["headers"] = headers
        kwargs["body"] = body

        resp = _APIRequestor.post(url, **kwargs)
        if resp.status_code >= 200 and resp.status_code < 300:
            return RecurringPayment(**resp.body)
        else:
            raise XenditError(resp)

    @staticmethod
    def resume_recurring_payment(
        *, id, x_idempotency_key=None, for_user_id=None, x_api_version=None, **kwargs,
    ):
        """Pause Recurring Payment (API Reference: Recurring Payment/Pause Recurring Payment)

        Args:
          - id (str)
          - **x_idempotency_key (str)
          - **for_user_id (str)
          - **x_api_version (str)

        Returns:
          RecurringPayment

        Raises:
          XenditError

        """
        url = f"/recurring_payments/{id}/resume!"
        headers, body = _extract_params(
            locals(),
            func_object=RecurringPayment.stop_recurring_payment,
            headers_params=["for_user_id", "x_idempotency_key", "x_api_version"],
            ignore_params=["id"],
        )
        kwargs["headers"] = headers
        kwargs["body"] = body

        resp = _APIRequestor.post(url, **kwargs)
        if resp.status_code >= 200 and resp.status_code < 300:
            return RecurringPayment(**resp.body)
        else:
            raise XenditError(resp)



from xendit.models._base_model import BaseModel


class CreditCardRefund(BaseModel):
    """Refund class (API Reference: Credit Card)

    Attributes:
      - updated (str)
      - created (str)
      - credit_card_charge_id (str)
      - user_id (str)
      - amount (int)
      - external_id (str)
      - status (str)
      - fee_refund_amount (int)
      - id (str)

    Optional Attributes:
      - failure_reason (str)
    """

    updated: str
    created: str
    credit_card_charge_id: str
    user_id: str
    amount: int
    external_id: str
    status: str
    fee_refund_amount: int
    id: str

    # Optional
    failure_reason: str



from xendit.models._base_model import BaseModel
from xendit.models._base_query import BaseQuery


class DirectDebitBasket(BaseModel):
    """Basket class in Direct Debit payment (API Reference: Direct Debit)

    Attributes:
      - reference_id (str)
      - name (str)
      - market (str)
      - type (str)
      - description (str)
      - category (str)
      - sub_category (str)
      - price (str)
      - url (str)
      - metadata (str)
      - quantity (str)
    """

    reference_id: str
    name: str
    market: str
    type: str
    description: str
    category: str
    sub_category: str
    price: str
    url: str
    metadata: str
    quantity: str

    class Query(BaseQuery):
        """Basket class in Direct Debit payment query (API Reference: Direct Debit)

        Use this for create_payment

        Attributes:
          - reference_id (str)
          - name (str)
          - market (str)
          - type (str)

        Optional Attributes:
          - description (str)
          - category (str)
          - sub_category (str)
          - price (str)
          - url (str)
          - metadata (str)
          - quantity (str)
        """

        reference_id: str
        name: str
        market: str
        type: str
        description: str
        category: str
        sub_category: str
        price: str
        url: str
        metadata: str
        quantity: str



from xendit.models._base_model import BaseModel
from xendit.models._base_query import BaseQuery


class CreditCardChargeInstallment(BaseModel):
    """Installment class of Charge (API Reference: Credit Card)

    Optional Attributes:
        - count (int)
        - interval (str)
    """

    # Optional
    count: int
    interval: str

    class Query(BaseQuery):
        """Installment class of Charge Query (API Reference: Credit Card)

        Used for create_authorization and create_charge

        Optional Attributes:
          - count (int)
          - interval (str)
        """

        # Optional
        count: int
        interval: str



import inspect


def _extract_params(function_locals, func_object, headers_params=[], ignore_params=[]):
    """Extract data from function_locals to headers and body of the request

    Args:
        - function_locals (dict): locals() of the function
        - func_object (function): Function that will be analyzed the parameter list
        - **headers_params (list): List of headers parameter. Defaults to [].
        - **ignore_params (list): List of params that won't be used on headers and body. Default to []

    Returns:
        dict, dict: headers and body of the request
    """
    body = dict()
    headers = dict()
    map_headers_key = {
        "for_user_id": "for-user-id",
        "x_api_version": "X-API-VERSION",
        "x_idempotency_key": "X-IDEMPOTENCY-KEY",
        "idempotency_key": "Idempotency-key",
    }
    for param in inspect.getfullargspec(func_object)[4]:
        value = function_locals.get(param, None)
        if value is not None and param not in ignore_params:
            if param in headers_params:
                mapped_param = map_headers_key[param]
                headers[mapped_param] = function_locals[param]
            else:
                body[param] = function_locals[param]
    return headers, body



from .virtual_acount_bank import VirtualAccountBank
from .virtual_account_payment import VirtualAccountPayment

from xendit.models._base_model import BaseModel

from xendit._api_requestor import _APIRequestor
from xendit._extract_params import _extract_params

from xendit.xendit_error import XenditError


class VirtualAccount(BaseModel):
    """Virtual Account class (API Reference: Virtual Account)

    Related Classes:
      - VirtualAccountBanks
      - VirtualAccountPayment

    Static Methods:
      - VirtualAccount.create (API Reference: /Create Virtual Account)
      - VirtualAccount.get_banks (API Reference: /Get Virtual Account Banks)
      - VirtualAccount.get (API Reference: /Get Virtual Account)
      - VirtualAccount.update (API Reference: /Update Virtual Account)
      - VirtualAccount.get_payment (API Reference: /Get Virtual Account Payment)

    Attributes:
      - owner_id (str)
      - external_id (str)
      - bank_code (str)
      - merchant_code (str)
      - name (str)
      - account_number (str)
      - is_closed (bool)
      - id (str)
      - is_single_use (bool)
      - status (str)
      - expiration_date (str) (ISO 8601 Date)

    Optional Attributes:
      - suggested_amount (str)
      - expected_amount (str)
      - description (str)

    """

    owner_id: str
    external_id: str
    bank_code: str
    merchant_code: str
    name: str
    account_number: str
    is_closed: bool
    id: str
    is_single_use: bool
    status: str
    expiration_date: str

    # Optional
    suggested_amount: str
    expected_amount: str
    description: str

    @staticmethod
    def create(
        *,
        external_id,
        bank_code,
        name,
        virtual_account_number=None,
        suggested_amount=None,
        is_closed=None,
        expected_amount=None,
        expiration_date=None,
        is_single_use=None,
        description=None,
        for_user_id=None,
        x_idempotency_key=None,
        x_api_version=None,
        **kwargs,
    ):
        """Send POST Request to create VirtualAccount (API Reference: Virtual Account/Create Virtual Account)

        Args:
          - external_id (str)
          - bank_code (str)
          - name (str)
          - **virtual_account_number (str)
          - **suggested_amount (int)
          - **is_closed (bool)
          - **expected_amount (int)
          - **expiration_date (str) (ISO 8601 Date)
          - **is_single_use (bool)
          - **description (str)
          - **for_user_id (str) (XenPlatforms only)
          - **x_idempotency_key (str)
          - **x_api_version (str): API Version that will be used. If not provided will default to the latest

        Returns:
          VirtualAccount

        Raises:
          XenditError

        """
        url = "/callback_virtual_accounts"
        headers, body = _extract_params(
            locals(),
            func_object=VirtualAccount.create,
            headers_params=["for_user_id", "x_idempotency_key", "x_api_version"],
        )
        kwargs["headers"] = headers
        kwargs["body"] = body

        resp = _APIRequestor.post(url, **kwargs)
        if resp.status_code >= 200 and resp.status_code < 300:
            return VirtualAccount(**resp.body)
        else:
            raise XenditError(resp)

    @staticmethod
    def get_banks(*, for_user_id=None, x_api_version=None, **kwargs):
        """Get available banks (API Reference: Virtual Account/Get Virtual Account Banks)

        Args:
          - **for_user_id (str) (XenPlatforms only)
          - **x_api_version (str): API Version that will be used. If not provided will default to the latest

        Returns:
          List of VirtualAccountBank

        Raises:
          XenditError

        """
        url = "/available_virtual_account_banks"
        headers, _ = _extract_params(
            locals(),
            func_object=VirtualAccount.get_banks,
            headers_params=["for_user_id", "x_api_version"],
            ignore_params=["id"],
        )
        kwargs["headers"] = headers

        resp = _APIRequestor.get(url, **kwargs)
        if resp.status_code >= 200 and resp.status_code < 300:
            virtual_account_banks = []
            for bank in resp.body:
                virtual_account_banks.append(VirtualAccountBank(**bank))
            return virtual_account_banks
        else:
            raise XenditError(resp)

    @staticmethod
    def get(*, id, for_user_id=None, x_api_version=None, **kwargs):
        """Get the detail of Virtual Account (API Reference: Virtual Account/Get Virtual Account

        Args:
          - id (str)
          - **for_user_id (str) (XenPlatforms only)
          - **x_api_version (str): API Version that will be used. If not provided will default to the latest

        Returns:
          VirtualAccount

        Raises:
          XenditError
        """
        url = f"/callback_virtual_accounts/{id}"
        headers, _ = _extract_params(
            locals(),
            func_object=VirtualAccount.get,
            headers_params=["for_user_id", "x_api_version"],
            ignore_params=["id"],
        )
        kwargs["headers"] = headers

        resp = _APIRequestor.get(url, **kwargs)
        if resp.status_code >= 200 and resp.status_code < 300:
            return VirtualAccount(**resp.body)
        else:
            raise XenditError(resp)

    @staticmethod
    def update(
        *,
        id,
        suggested_amount=None,
        expected_amount=None,
        expiration_date=None,
        is_single_use=None,
        description=None,
        for_user_id=None,
        x_idempotency_key=None,
        x_api_version=None,
        **kwargs,
    ):
        """Update Virtual Account detail (API Reference: Virtual Account/Update Virtual Account

        Args:
          - id (str)
          - **suggested_amount (int)
          - **expected_amount (int)
          - **expiration_date (str) (ISO 8601 Date)
          - **is_single_use (bool)
          - **description (str)
          - **for_user_id (str) (XenPlatforms only)
          - **x_idempotency_key (str)
          - **x_api_version (str): API Version that will be used. If not provided will default to the latest

        Returns:
          VirtualAccount

        Raises:
          XenditError
        """
        url = f"/callback_virtual_accounts/{id}"
        headers, body = _extract_params(
            locals(),
            func_object=VirtualAccount.update,
            headers_params=["x_idempotency_key", "for_user_id", "x_api_version"],
            ignore_params=["id"],
        )
        kwargs["headers"] = headers
        kwargs["body"] = body

        resp = _APIRequestor.patch(url, **kwargs)
        if resp.status_code >= 200 and resp.status_code < 300:
            return VirtualAccount(**resp.body)
        else:
            raise XenditError(resp)

    @staticmethod
    def get_payment(*, payment_id, for_user_id=None, x_api_version=None, **kwargs):
        """Get payment from virtual account (API Reference: Virtual Account/Get Virtual Account Payment

        Args:
          - payment_id (str)

        Returns:
          VirtualAccountPayment

        Raises:
          XenditError
        """
        url = f"/callback_virtual_account_payments/payment_id={payment_id}"
        headers, _ = _extract_params(
            locals(),
            func_object=VirtualAccount.get_payment,
            headers_params=["for_user_id", "x_api_version"],
            ignore_params=["payment_id"],
        )
        kwargs["headers"] = headers

        resp = _APIRequestor.get(url, **kwargs)
        if resp.status_code >= 200 and resp.status_code < 300:
            return VirtualAccountPayment(**resp.body)
        else:
            raise XenditError(resp)



from typing import Union

from .direct_debit_card_link import DirectDebitCardLink
from .direct_debit_online_banking_link import DirectDebitOnlineBankingLink

from xendit.models._base_model import BaseModel


class DirectDebitAccessibleAccount(BaseModel):
    """Accessible Account in Direct Debit (API Reference: Direct Debit)

    Attributes:
      - id (str)
      - channel_code (str)
      - type (str)
      - properties (DirectDebitCardLink or DirectDebitOnlineBankingLink)

    """

    id: str
    channel_code: str
    type: str
    properties: Union[DirectDebitCardLink, DirectDebitOnlineBankingLink]



from xendit.models._base_model import BaseModel


class OVOPayment(BaseModel):
    """Payment detail for OVO (API Reference: eWallets)

    Attributes:
      - amount (int)
      - business_id (str)
      - created (str) (ISO 8601)
      - ewallet_type (str)
      - external_id (str)
      - phone (str)
      - status (str)

    Legacy Attributes:
      - transaction_date (Last version: 2019-02-04)
      - ewallet_transaction_id (Last version: 2019-02-04)
    """

    amount: int
    business_id: str
    created: str
    ewallet_type: str
    external_id: str
    phone: str
    status: str

    # Attributes for (x_api_version == 2019-02-04)
    transaction_date: str
    ewallet_transaction_id: str



import json
from dataclasses import dataclass


@dataclass(init=False)
class BaseModel:
    """Abstract class for feature class. Useful for pretty print and automatically set attribute"""

    def __init__(self, **kwargs):
        for key, value in kwargs.items():
            setattr(self, key, value)

    def __repr__(self):
        return json.dumps(vars(self), indent=4)



from xendit.models._base_model import BaseModel
from xendit.models._base_query import BaseQuery


class LinkAjaItem(BaseModel):
    """Item Class for LinkAja (API Reference: eWallets)"""

    class Query(BaseQuery):
        """Item Object for LinkAja Query (API Reference: eWallets)

        Use this for create_linkaja_payment

        Attributes:
          - id (str)
          - name (str)
          - price (int)
          - quantity (int)
        """

        id: str
        name: str
        price: int
        quantity: int



from xendit.models._base_model import BaseModel


class InvoiceRetailOutlet(BaseModel):
    """EWallet data detail in Invoice (API Reference: Invoice)

    Attributes:
      - ewallet_type (str)
    """

    ewallet_type: str



class XenditError(Exception):
    """Error that will be given when status code != 200."""

    def __init__(self, xendit_response):
        try:
            super(XenditError, self).__init__(xendit_response.body["message"])
        except KeyError:
            super(XenditError, self).__init__(
                xendit_response.body["errors"][0]["message"]
            )
        self.status_code = xendit_response.status_code
        self.error_code = xendit_response.body["error_code"]
        self.errors = xendit_response.body.get("errors", None)



from xendit.models._base_model import BaseModel


class DisbursementBank(BaseModel):
    """Bank class for Disbursement (API Reference: Disbursement)

    Attributes:
      - name (str)
      - code (str)
      - can_disburse (bool)
      - can_name_validate (bool)
    """

    name: str
    code: str
    can_disburse: bool
    can_name_validate: bool



from xendit.models._base_model import BaseModel
from xendit.models._base_query import BaseQuery


class DirectDebitPaymentMethodProperties(BaseModel):
    """Payment Method Properties class in Direct Debit (API Reference: Direct Debit)

    Attributes:
      - id (str)

    """

    class Query(BaseQuery):
        """Payment Method Properties class in Direct Debit Query (API Reference: Direct Debit)

        Attributes:
        - id (str)

        """

        id: str



from xendit.models._base_model import BaseModel


class LinkAjaPayment(BaseModel):
    """Payment detail for LinkAja (API Reference: eWallets)

    Attributes:
      - checkout_url (str)
      - transaction_date (str) (ISO 8601)
      - amount (int)
      - external_id (str)
      - ewallet_type (str)
    """

    checkout_url: str
    transaction_date: str
    amount: int
    external_id: str
    ewallet_type: str
